"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.injectAsyncStore = injectAsyncStore;
exports.rejectAsyncStore = rejectAsyncStore;
exports.default = initStore;

var _assign = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/assign"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/promise"));

var _interopRequireWildcard2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/interopRequireWildcard"));

var _redux = require("redux");

var _reduxLogger = require("redux-logger");

var _reduxSaga = _interopRequireDefault(require("redux-saga"));

var _reducers = _interopRequireDefault(require("./reducers"));

const logger = (0, _reduxLogger.createLogger)();
const win = window;

function injectAsyncStore(store, asyncReducers, sagas) {
  asyncReducers && injectAsyncReducers(store, asyncReducers);
  sagas && injectAsyncSagas(store, sagas);
}

function injectAsyncReducers(store, asyncReducers) {
  let flag = false;

  for (const key in asyncReducers) {
    if (Object.prototype.hasOwnProperty.call(asyncReducers, key)) {
      if (!store.asyncReducers[key]) {
        store.asyncReducers[key] = asyncReducers[key];
        flag = true;
      }
    }
  }

  flag && store.replaceReducer((0, _reducers.default)(store.asyncReducers));
}

function injectAsyncSagas(store, sagas) {
  for (const key in sagas) {
    if (Object.prototype.hasOwnProperty.call(sagas, key)) {
      if (!store.asyncSagas[key]) {
        store.asyncSagas[key] = sagas[key];
        store.sagaMiddleware.run(sagas[key]);
      }
    }
  }
}

function rejectAsyncStore(store, asyncReducers = null) {
  asyncReducers && rejectAsyncReducers(store, asyncReducers);
}

function rejectAsyncReducers(store, asyncReducers) {
  let flag = false;

  for (const key in asyncReducers) {
    if (Object.prototype.hasOwnProperty.call(asyncReducers, key)) {
      if (store.asyncReducers[key]) {
        delete store.asyncReducers[key];
        flag = true;
      }
    }
  }

  flag && store.replaceReducer((0, _reducers.default)(store.asyncReducers));
}

function initStore(initReducer = {}, reduxMiddlewares = []) {
  const sagaMiddleware = (0, _reduxSaga.default)();
  const middlewares = [...reduxMiddlewares, sagaMiddleware];

  if (process.env.NODE_ENV !== 'production') {
    _promise.default.resolve().then(() => (0, _interopRequireWildcard2.default)(require('redux-immutable-state-invariant'))).then(({
      default: _
    }) => {
      middlewares.push(_());
    });
  }

  const storeEnhancers = (0, _redux.compose)((0, _redux.applyMiddleware)(...middlewares, logger), win && win.__REDUX_DEVTOOLS_EXTENSION__ ? win.__REDUX_DEVTOOLS_EXTENSION__() : f => f);
  const store = (0, _redux.createStore)((0, _reducers.default)(initReducer), storeEnhancers);
  return (0, _assign.default)({}, store, {
    asyncReducers: {},
    asyncSagas: {},
    sagaMiddleware
  });
}